@@ -1,5 +1,5 @@
-# parsetree.py
-# Copyright (C) 2006, 2007, 2008, 2009, 2010 Michael Bayer mike_mp@zzzcomputing.com
+# mako/parsetree.py
+# Copyright (C) 2006-2011 the Mako authors and contributors <see AUTHORS file>
 #
 # This module is part of Mako and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -16,14 +16,15 @@ class Node(object):
         self.lineno = lineno
         self.pos = pos
         self.filename = filename
-    
+ 
+    @property
     def exception_kwargs(self):
-        return {'source':self.source, 'lineno':self.lineno, 'pos':self.pos, 'filename':self.filename}
-    exception_kwargs = property(exception_kwargs)
-    
+        return {'source':self.source, 'lineno':self.lineno, 
+                'pos':self.pos, 'filename':self.filename}
+ 
     def get_children(self):
         return []
-        
+ 
     def accept_visitor(self, visitor):
         def traverse(node):
             for n in node.get_children():
@@ -33,27 +34,29 @@ class Node(object):
 
 class TemplateNode(Node):
     """a 'container' node that stores the overall collection of nodes."""
-    
+ 
     def __init__(self, filename):
         super(TemplateNode, self).__init__('', 0, 0, filename)
         self.nodes = []
         self.page_attributes = {}
-        
+ 
     def get_children(self):
         return self.nodes
-        
+ 
     def __repr__(self):
-        return "TemplateNode(%s, %r)" % (util.sorted_dict_repr(self.page_attributes), self.nodes)
-        
+        return "TemplateNode(%s, %r)" % (
+                    util.sorted_dict_repr(self.page_attributes), 
+                    self.nodes)
+ 
 class ControlLine(Node):
     """defines a control line, a line-oriented python line or end tag.
-    
+ 
     e.g.::
 
         % if foo:
             (markup)
         % endif
-    
+ 
     """
 
     def __init__(self, keyword, isend, text, **kwargs):
@@ -75,16 +78,17 @@ class ControlLine(Node):
 
     def undeclared_identifiers(self):
         return self._undeclared_identifiers
-        
+ 
     def is_ternary(self, keyword):
-        """return true if the given keyword is a ternary keyword for this ControlLine"""
-        
+        """return true if the given keyword is a ternary keyword
+        for this ControlLine"""
+ 
         return keyword in {
             'if':set(['else', 'elif']),
             'try':set(['except', 'finally']),
             'for':set(['else'])
         }.get(self.keyword, [])
-        
+ 
     def __repr__(self):
         return "ControlLine(%r, %r, %r, %r)" % (
             self.keyword, 
@@ -95,29 +99,29 @@ class ControlLine(Node):
 
 class Text(Node):
     """defines plain text in the template."""
-    
+ 
     def __init__(self, content, **kwargs):
         super(Text, self).__init__(**kwargs)
         self.content = content
-        
+ 
     def __repr__(self):
         return "Text(%r, %r)" % (self.content, (self.lineno, self.pos))
-        
+ 
 class Code(Node):
     """defines a Python code block, either inline or module level.
-    
+ 
     e.g.::
 
         inline:
         <%
             x = 12
         %>
-    
+ 
         module level:
         <%!
             import logger
         %>
-    
+ 
     """
 
     def __init__(self, text, ismodule, **kwargs):
@@ -138,28 +142,28 @@ class Code(Node):
             self.ismodule, 
             (self.lineno, self.pos)
         )
-        
+ 
 class Comment(Node):
     """defines a comment line.
-    
+ 
     # this is a comment
-    
+ 
     """
-    
+ 
     def __init__(self, text, **kwargs):
         super(Comment, self).__init__(**kwargs)
         self.text = text
 
     def __repr__(self):
         return "Comment(%r, %r)" % (self.text, (self.lineno, self.pos))
-        
+ 
 class Expression(Node):
     """defines an inline expression.
-    
+ 
     ${x+y}
-    
+ 
     """
-    
+ 
     def __init__(self, text, escapes, **kwargs):
         super(Expression, self).__init__(**kwargs)
         self.text = text
@@ -176,7 +180,7 @@ class Expression(Node):
                 self.escapes_code.undeclared_identifiers.difference(
                     set(filters.DEFAULT_ESCAPES.keys())
                 )
-            )
+            ).difference(self.code.declared_identifiers)
 
     def __repr__(self):
         return "Expression(%r, %r, %r)" % (
@@ -184,21 +188,23 @@ class Expression(Node):
             self.escapes_code.args, 
             (self.lineno, self.pos)
         )
-        
+ 
 class _TagMeta(type):
-    """metaclass to allow Tag to produce a subclass according to its keyword"""
-    
+    """metaclass to allow Tag to produce a subclass according to
+    its keyword"""
+ 
     _classmap = {}
-    
+ 
     def __init__(cls, clsname, bases, dict):
         if cls.__keyword__ is not None:
             cls._classmap[cls.__keyword__] = cls
             super(_TagMeta, cls).__init__(clsname, bases, dict)
-            
+ 
     def __call__(cls, keyword, attributes, **kwargs):
         if ":" in keyword:
             ns, defname = keyword.split(':')
-            return type.__call__(CallNamespaceTag, ns, defname, attributes, **kwargs)
+            return type.__call__(CallNamespaceTag, ns, defname, 
+                                        attributes, **kwargs)
 
         try:
             cls = _TagMeta._classmap[keyword]
@@ -211,38 +217,41 @@ class _TagMeta(type):
                 filename=kwargs['filename']
             )
         return type.__call__(cls, keyword, attributes, **kwargs)
-        
+ 
 class Tag(Node):
     """abstract base class for tags.
-    
+ 
     <%sometag/>
-    
+ 
     <%someothertag>
         stuff
     </%someothertag>
-    
+ 
     """
-    
+ 
     __metaclass__ = _TagMeta
     __keyword__ = None
-    
-    def __init__(self, keyword, attributes, expressions, nonexpressions, required, **kwargs):
+ 
+    def __init__(self, keyword, attributes, expressions, 
+                        nonexpressions, required, **kwargs):
         """construct a new Tag instance.
-        
-        this constructor not called directly, and is only called by subclasses.
-        
-        keyword - the tag keyword
-        
-        attributes - raw dictionary of attribute key/value pairs
-        
-        expressions - a set of identifiers that are legal attributes, 
-            which can also contain embedded expressions
-        
-        nonexpressions - a set of identifiers that are legal attributes, 
-            which cannot contain embedded expressions
-        
-        \**kwargs - other arguments passed to the Node superclass (lineno, pos)
-        
+ 
+        this constructor not called directly, and is only called
+        by subclasses.
+ 
+        :param keyword: the tag keyword
+ 
+        :param attributes: raw dictionary of attribute key/value pairs
+ 
+        :param expressions: a set of identifiers that are legal attributes, 
+         which can also contain embedded expressions
+ 
+        :param nonexpressions: a set of identifiers that are legal 
+         attributes, which cannot contain embedded expressions
+ 
+        :param \**kwargs:
+         other arguments passed to the Node superclass (lineno, pos)
+ 
         """
         super(Tag, self).__init__(**kwargs)
         self.keyword = keyword
@@ -251,17 +260,18 @@ class Tag(Node):
         missing = [r for r in required if r not in self.parsed_attributes]
         if len(missing):
             raise exceptions.CompileException(
-                "Missing attribute(s): %s" % ",".join([repr(m) for m in missing]), 
+                "Missing attribute(s): %s" % 
+                    ",".join([repr(m) for m in missing]), 
                 **self.exception_kwargs)
         self.parent = None
         self.nodes = []
-        
+ 
     def is_root(self):
         return self.parent is None
-        
+ 
     def get_children(self):
         return self.nodes
-        
+ 
     def _parse_attributes(self, expressions, nonexpressions):
         undeclared_identifiers = set()
         self.parsed_attributes = {}
@@ -274,6 +284,10 @@ class Tag(Node):
                     if m:
                         code = ast.PythonCode(m.group(1).rstrip(),
                                 **self.exception_kwargs)
+                        # we aren't discarding "declared_identifiers" here,
+                        # which we do so that list comprehension-declared 
+                        # variables aren't counted.   As yet can't find a 
+                        # condition that requires it here.
                         undeclared_identifiers = \
                             undeclared_identifiers.union(
                                     code.undeclared_identifiers)
@@ -304,12 +318,12 @@ class Tag(Node):
 
     def __repr__(self):
         return "%s(%r, %s, %r, %r)" % (self.__class__.__name__, 
-                                        self.keyword, 
-                                        util.sorted_dict_repr(self.attributes),
-                                        (self.lineno, self.pos), 
-                                        self.nodes
-                                    )
-        
+                                    self.keyword, 
+                                    util.sorted_dict_repr(self.attributes),
+                                    (self.lineno, self.pos), 
+                                    self.nodes
+                                )
+ 
 class IncludeTag(Tag):
     __keyword__ = 'include'
 
@@ -327,9 +341,12 @@ class IncludeTag(Tag):
         return []
 
     def undeclared_identifiers(self):
-        identifiers = self.page_args.undeclared_identifiers.difference(set(["__DUMMY"]))
-        return identifiers.union(super(IncludeTag, self).undeclared_identifiers())
-    
+        identifiers = self.page_args.undeclared_identifiers.\
+                            difference(set(["__DUMMY"])).\
+                            difference(self.page_args.declared_identifiers)
+        return identifiers.union(super(IncludeTag, self).
+                                    undeclared_identifiers())
+ 
 class NamespaceTag(Tag):
     __keyword__ = 'namespace'
 
@@ -340,13 +357,18 @@ class NamespaceTag(Tag):
                                         ('name','inheritable',
                                         'import','module'), 
                                         (), **kwargs)
-                                        
+ 
         self.name = attributes.get('name', '__anon_%s' % hex(abs(id(self))))
         if not 'name' in attributes and not 'import' in attributes:
             raise exceptions.CompileException(
-                                "'name' and/or 'import' attributes are required "
-                                "for <%namespace>", 
-                                **self.exception_kwargs)
+                "'name' and/or 'import' attributes are required "
+                "for <%namespace>",
+                **self.exception_kwargs)
+        if 'file' in attributes and 'module' in attributes:
+            raise exceptions.CompileException(
+                "<%namespace> may only have one of 'file' or 'module'",
+                **self.exception_kwargs
+            )
 
     def declared_identifiers(self):
         return []
@@ -362,7 +384,7 @@ class TextTag(Tag):
         self.filter_args = ast.ArgumentList(
                                     attributes.get('filter', ''), 
                                     **self.exception_kwargs)
-        
+ 
 class DefTag(Tag):
     __keyword__ = 'def'
 
@@ -380,7 +402,8 @@ class DefTag(Tag):
             raise exceptions.CompileException(
                                 "Missing parenthesis in %def", 
                                 **self.exception_kwargs)
-        self.function_decl = ast.FunctionDecl("def " + name + ":pass", **self.exception_kwargs)
+        self.function_decl = ast.FunctionDecl("def " + name + ":pass", 
+                                                    **self.exception_kwargs)
         self.name = self.function_decl.funcname
         self.decorator = attributes.get('decorator', '')
         self.filter_args = ast.ArgumentList(
@@ -408,13 +431,15 @@ class CallTag(Tag):
                                     ('args'), ('expr',), ('expr',), **kwargs)
         self.expression = attributes['expr']
         self.code = ast.PythonCode(self.expression, **self.exception_kwargs)
-        self.body_decl = ast.FunctionArgs(attributes.get('args', ''), **self.exception_kwargs)
+        self.body_decl = ast.FunctionArgs(attributes.get('args', ''), 
+                                            **self.exception_kwargs)
 
     def declared_identifiers(self):
         return self.code.declared_identifiers.union(self.body_decl.argnames)
 
     def undeclared_identifiers(self):
-        return self.code.undeclared_identifiers
+        return self.code.undeclared_identifiers.\
+                    difference(self.code.declared_identifiers)
 
 class CallNamespaceTag(Tag):
 
@@ -426,7 +451,7 @@ class CallNamespaceTag(Tag):
                     (), 
                     (), 
                     **kwargs)
-                    
+ 
         self.expression = "%s.%s(%s)" % (
                                 namespace, 
                                 defname, 
@@ -443,7 +468,8 @@ class CallNamespaceTag(Tag):
         return self.code.declared_identifiers.union(self.body_decl.argnames)
 
     def undeclared_identifiers(self):
-        return self.code.undeclared_identifiers
+        return self.code.undeclared_identifiers.\
+                    difference(self.code.declared_identifiers)
 
 class InheritTag(Tag):
     __keyword__ = 'inherit'
@@ -466,12 +492,13 @@ class PageTag(Tag):
                 (), 
                 (), 
                 **kwargs)
-        self.body_decl = ast.FunctionArgs(attributes.get('args', ''), **self.exception_kwargs)
+        self.body_decl = ast.FunctionArgs(attributes.get('args', ''), 
+                                            **self.exception_kwargs)
         self.filter_args = ast.ArgumentList(
                                 attributes.get('expression_filter', ''),
                                 **self.exception_kwargs)
 
     def declared_identifiers(self):
         return self.body_decl.argnames
-        
-    
+ 
+ 
